import { Model } from "../../model";
export class RandomGenerator extends Model {
    constructor(attrs) {
        super(attrs);
    }
}
RandomGenerator.__name__ = "RandomGenerator";
//# sourceMappingURL=random_generator.js.map