import { Icon, IconView } from "./icon";
import { StyleSheet, StyleSheetLike } from "../../../core/dom";
import * as p from "../../../core/properties";
export declare class SVGIconView extends IconView {
    model: SVGIcon;
    protected readonly _style: StyleSheet;
    styles(): StyleSheetLike[];
    render(): void;
}
export declare namespace SVGIcon {
    type Attrs = p.AttrsOf<Props>;
    type Props = Icon.Props & {
        svg: p.Property<string>;
    };
}
export interface SVGIcon extends SVGIcon.Attrs {
}
export declare class SVGIcon extends Icon {
    properties: SVGIcon.Props;
    __view_type__: SVGIconView;
    constructor(attrs?: Partial<SVGIcon.Attrs>);
}
//# sourceMappingURL=svg_icon.d.ts.map