var _a;
import { Model } from "../../model";
export class Selector extends Model {
    constructor(attrs) {
        super(attrs);
    }
}
_a = Selector;
Selector.__name__ = "Selector";
(() => {
    _a.define(({ String }) => ({
        query: [String],
    }));
})();
//# sourceMappingURL=selector.js.map