export const non_interactive = "bk-non-interactive"
export const tooltip_content = "bk-tooltip-content"
export const left = "bk-left"
export const tooltip_arrow = "bk-tooltip-arrow"
export const right = "bk-right"
export const above = "bk-above"
export const below = "bk-below"
export const tooltip_row_label = "bk-tooltip-row-label"
export const tooltip_row_value = "bk-tooltip-row-value"
export const tooltip_color_block = "bk-tooltip-color-block"
export const close = "bk-close"
export default `:host{--tooltip-border:#e5e5e5;--tooltip-color:white;--tooltip-text:#2f2f2f;--tooltip-arrow-color:#909599;--tooltip-arrow-width:10px;--tooltip-arrow-height:10px;--tooltip-arrow-half-width:7px;--tooltip-arrow-half-height:7px;}:host{width:max-content;font-weight:300;font-size:12px;position:absolute;padding:5px;border:1px solid var(--tooltip-border);color:var(--tooltip-text);background-color:var(--tooltip-color);opacity:0.95;z-index:100;}:host(.bk-non-interactive){pointer-events:none;}.bk-tooltip-content > div:not(:first-child){margin-top:5px;border-top:var(--tooltip-border) 1px dashed;}:host(.bk-left.bk-tooltip-arrow) .bk-tooltip-content::before{position:absolute;margin:calc(-1*var(--tooltip-arrow-half-height)) 0 0 0;top:50%;width:0;height:0;border-style:solid;border-width:var(--tooltip-arrow-half-height) 0 var(--tooltip-arrow-half-height) 0;border-color:transparent;content:" ";display:block;left:calc(-1*var(--tooltip-arrow-width));border-right-width:var(--tooltip-arrow-width);border-right-color:var(--tooltip-arrow-color);}:host(.bk-left) .bk-tooltip-content::before{left:calc(-1*var(--tooltip-arrow-width));border-right-width:var(--tooltip-arrow-width);border-right-color:var(--tooltip-arrow-color);}:host(.bk-right.bk-tooltip-arrow) .bk-tooltip-content::after{position:absolute;margin:calc(-1*var(--tooltip-arrow-half-height)) 0 0 0;top:50%;width:0;height:0;border-style:solid;border-width:var(--tooltip-arrow-half-height) 0 var(--tooltip-arrow-half-height) 0;border-color:transparent;content:" ";display:block;right:calc(-1*var(--tooltip-arrow-width));border-left-width:var(--tooltip-arrow-width);border-left-color:var(--tooltip-arrow-color);}:host(.bk-right) .bk-tooltip-content::after{right:calc(-1*var(--tooltip-arrow-width));border-left-width:var(--tooltip-arrow-width);border-left-color:var(--tooltip-arrow-color);}:host(.bk-above) .bk-tooltip-content::before{position:absolute;margin:0 0 0 calc(-1*var(--tooltip-arrow-half-width));left:50%;width:0;height:0;border-style:solid;border-width:0 var(--tooltip-arrow-half-width) 0 var(--tooltip-arrow-half-width);border-color:transparent;content:" ";display:block;top:calc(-1*var(--tooltip-arrow-height));border-bottom-width:var(--tooltip-arrow-height);border-bottom-color:var(--tooltip-arrow-color);}:host(.bk-below) .bk-tooltip-content::after{position:absolute;margin:0 0 0 calc(-1*var(--tooltip-arrow-half-width));left:50%;width:0;height:0;border-style:solid;border-width:0 var(--tooltip-arrow-half-width) 0 var(--tooltip-arrow-half-width);border-color:transparent;content:" ";display:block;bottom:calc(-1*var(--tooltip-arrow-height));border-top-width:var(--tooltip-arrow-height);border-top-color:var(--tooltip-arrow-color);}.bk-tooltip-row-label{text-align:right;color:#26aae1;}.bk-tooltip-row-value{color:none;}.bk-tooltip-color-block{width:12px;height:12px;margin-left:5px;margin-right:5px;outline:#dddddd solid 1px;display:inline-block;}.bk-close{position:absolute;top:2px;right:2px;width:12px;height:12px;cursor:pointer;background-color:gray;mask-image:var(--bokeh-icon-x);mask-size:contain;mask-repeat:no-repeat;-webkit-mask-image:var(--bokeh-icon-x);-webkit-mask-size:contain;-webkit-mask-repeat:no-repeat;}.bk-close:hover{background-color:red;}`
