export { MapOptions } from "./gmap_plot";
export { GMapOptions } from "./gmap_plot";
export { GMapPlot } from "./gmap_plot";
export { GMap } from "./gmap";
export { Plot } from "./plot";
export { GridPlot } from "./grid_plot";
export { Figure } from "./figure";
//# sourceMappingURL=index.d.ts.map