export const layer = "bk-layer"
export const events = "bk-events"
export default `.bk-layer{position:absolute;top:0;left:0;width:100%;height:100%;}.bk-layer.bk-events{user-select:none;-webkit-user-select:none;touch-action:auto;-webkit-user-drag:none;-webkit-tap-highlight-color:rgba(0, 0, 0, 0);}`
