import { NDArray } from "../core/util/ndarray";
import { Numerical } from "./linalg";
export declare function f(strings: TemplateStringsArray, ...subs: number[]): number;
export declare function f(strings: TemplateStringsArray, ...subs: Numerical[]): NDArray;
//# sourceMappingURL=expr.d.ts.map