import { Model } from "../../model";
import { View } from "../../core/view";
export class ActionView extends View {
}
ActionView.__name__ = "ActionView";
export class Action extends Model {
    constructor(attrs) {
        super(attrs);
    }
}
Action.__name__ = "Action";
Action.__module__ = "bokeh.models.dom";
//# sourceMappingURL=action.js.map